/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2014
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

package de.unihalle.informatik.Alida.dataio.provider.swing.components;

import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;

import javax.swing.*;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerSwing;
import de.unihalle.informatik.Alida.dataio.provider.swing.events.*;

/**
 * Alida-specific Swing component linked to a {@link JComboBox}.
 * <p>
 * This component wraps a {@link JComboBox} thereby triggering value change
 * events and adding support for tooltips.
 * 
 * @author moeller
 */
public class ALDSwingComponentComboBox extends ALDSwingComponent 
	implements ItemListener {
	
	/**
	 * Associated Swing component.
	 */
	protected JComboBox compComboBox = null;

	/**
	 * Default constructor.
	 * @param its	Set of items to be shown in combobox.
	 */
	public ALDSwingComponentComboBox(	Vector<ALDSwingComponentComboBoxItem> its){
		this.compComboBox = new JComboBox(its);
		this.compComboBox.addItemListener(this);
		// add our own tooltip renderer
		this.compComboBox.setRenderer(new ComboBoxTooltipRenderer(its));
	}

	@Override
	public JComboBox getJComponent() {
		return this.compComboBox;
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		if (   e.getStateChange() == ItemEvent.SELECTED 
				&& ALDDataIOManagerSwing.getInstance().isTriggerValueChangeEvents()) {
			this.fireALDSwingValueChangeEvent(new ALDSwingValueChangeEvent(this));
		}
	}
	
	/**
	 * Selects the item associated with the given object.
	 * @param obj		Object which should be selected.
	 */
	public void setSelectedItem(Object obj) {
		int itemNum = this.compComboBox.getItemCount();
		for (int i=0;i<itemNum;++i) {
			Object item = this.compComboBox.getItemAt(i);
			ALDSwingComponentComboBoxItem boxItem =
					(ALDSwingComponentComboBoxItem)item;
			// if value is null, set selection to first entry 
			if (obj == null) {
				this.compComboBox.setSelectedIndex(0);
				return;
			}
			// if tooltip text is given, use this for item matching
			if (boxItem.getTooltip() != null) {
				if (obj.getClass().getCanonicalName().equals(boxItem.getTooltip())) {
					this.compComboBox.setSelectedIndex(i);
					return;
				}
			}
			// otherwise use item text directly for matching
			else if (obj.toString().equals(boxItem.getItemText())) {
				this.compComboBox.setSelectedIndex(i);
				return;
			}
		}
	}
	
	/**
	 * Tooltip renderer class, adds a tooltip to each item in the combobox.
	 * <p>
	 * Source code originally taken from here:
	 * @see <a href="http://www.java2s.com/Code/Java/Swing-Components/ToolTipComboBoxExample.htm">Java Swing documentation</a>
	 * 
	 * @author moeller
	 */
	protected class ComboBoxTooltipRenderer extends BasicComboBoxRenderer {
		
		/**
		 * List of items represented by combobox.
		 */
		private Vector<ALDSwingComponentComboBoxItem> items = null;
		
		/**
		 * Default constructor.
		 * @param itms	Set of combobox item objects.
		 */
		public ComboBoxTooltipRenderer(
														Vector<ALDSwingComponentComboBoxItem> itms){
			this.items = itms;
		}
		
    @Override
    public Component getListCellRendererComponent(JList list, Object value,
        int index, boolean isSelected, boolean cellHasFocus) {
      if (isSelected) {
        setBackground(list.getSelectionBackground());
        setForeground(list.getSelectionForeground());
        if (  this.items != null 
        		&& -1 < index	&& index < this.items.size()) {
          list.setToolTipText(this.items.get(index).getTooltip());
        }
      } else {
        setBackground(list.getBackground());
        setForeground(list.getForeground());
      }
      setFont(list.getFont());
      setText((value == null) ? "" : value.toString());
      return this;
    }
  }

	@Override
  public void disableComponent() {
		this.compComboBox.setEnabled(false);
  }

	@Override
  public void enableComponent() {
		this.compComboBox.setEnabled(true);
  }
	
	@Override
  public void dispose() {
		// nothing to do here
  }
}